local HideBlizzard = LibStub("AceAddon-3.0"):NewAddon("HideBlizzard", "AceConsole-3.0", "AceEvent-3.0")
local CURRENT_VERSION = GetAddOnMetadata("HideBlizzard", "Version")

local format = string.format

local defaults = {
	profile = {
		modules = {
			About = true,
			ActionBar = false,
			Buttons = false,
			Pet = false,
			Player = false,
			SpecialFrames = false,
			Target = false,
			Vehicle = false,
		},
	}
}

HideBlizzard:SetDefaultModuleState(false)
HideBlizzard:SetDefaultModuleLibraries("AceEvent-3.0")

function HideBlizzard:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HideBlizzardDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateView")
	db = self.db.profile

	self:RegisterOptions()
	self:RegisterCommands()

	StaticPopupDialogs["RESET_CONFIRM"] = {
		text = "Are you REALLY sure you want to reset the database? This will also reload the interface.",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			wipe(HideBlizzardDB)
			ReloadUI()
		end,
		OnCancel = function() StaticPopup_Hide("RESET_CONFIRM") end,
		showAlert = 1,
		exclusive = 1,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
	}
end

function HideBlizzard:OnEnable()
	self:Welcome()
	self:Update()
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
end

function HideBlizzard:OnDisable()
end

function HideBlizzard:UpdateView()
	for k, v in self:IterateModules() do
		if self:GetModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k)
		elseif not self:GetModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k)
		end
		if type(v.UpdateView) == "function" then
			v:UpdateView()
		end
	end
end

function HideBlizzard:PLAYER_REGEN_DISABLED()
	if InterfaceOptionsFrame:IsVisible() then
		self:Print("Cannot configure settings while in combat. Closing options window.")
		InterfaceOptionsFrame:Hide()
	end
end

function HideBlizzard:Welcome()
	db = self.db.profile
	local lastUpdate = 0
	if not db.Welcome then
		delay = delay or CreateFrame("Frame")
		delay:SetScript("OnUpdate", function(self, elapsed)
			lastUpdate = lastUpdate + elapsed
			if (lastUpdate > 5) then
				HideBlizzard:Print("Welcome! Type|r |cff40bbff/hb|r or |cff40bbff/hideblizzard|r to see the list of available commands.")
				HideBlizzard.db.profile.Welcome = true
				HideBlizzard.db.profile.Version = CURRENT_VERSION
				lastUpdate = 0
				delay:SetScript("OnUpdate", nil)
			end
		end)
	end
end

function HideBlizzard:Update()
	db = self.db.profile
	local lastUpdate = 0
	if db.Version ~=  CURRENT_VERSION then
		if not db.Welcome and not db.Version then
			return
		else
			delay = delay or CreateFrame("Frame")
			delay:SetScript("OnUpdate", function(self, elapsed)
				lastUpdate = lastUpdate + elapsed
				if (lastUpdate > 5) then
					HideBlizzard.db.profile.Version = CURRENT_VERSION
					HideBlizzard:Print(format("Updated to version %s", HideBlizzard.db.profile.Version))
					HideBlizzard:ShowChangelog()
					lastUpdate = 0
					delay:SetScript("OnUpdate", nil)
				end
			end)
		end
	end
end

function HideBlizzard:RegisterCommands()
	db = self.db.profile
	function HideBlizzard:Cmd(args)
		local cmd = string.split(" ", args):lower() or args:lower()
		if cmd == "changelog" then
			self:ShowChangelog()
		elseif cmd == "config" then
			InterfaceOptionsFrame_OpenToCategory(HideBlizzard.optionFrame.Profiles)
			InterfaceOptionsFrame_OpenToCategory(HideBlizzard.optionFrame.HideBlizzard)
		elseif cmd == "help" then
			self:ShowHelp()
		elseif cmd == "reset" then
			StaticPopup_Show("RESET_CONFIRM")
		elseif cmd == "version" then
			self:Print(format("Your current version is |cffff0000%s|r", db.Version))
		else
			self:CmdHelp()
		end
	end

	function HideBlizzard:CmdHelp(cmd)
		local function PrintCmd(cmd, desc)
			DEFAULT_CHAT_FRAME:AddMessage(format(" - |cff40bbff%s|r: %s", cmd, desc))
		end
		self:Print("Available Commands")
		PrintCmd("changelog", "opens the changlog window")
		PrintCmd("config", "opens the option window")
		PrintCmd("help", "opens the help window")
		PrintCmd("reset", "resets the HideBlizzard database")
		PrintCmd("version", "prints the current version")
	end

	self:RegisterChatCommand("hb", "Cmd")
	self:RegisterChatCommand("hideblizzard", "Cmd")
end

function HideBlizzard:CheckClass(compared)
	local _, player = UnitClass("player")
	if player == compared then
		return false
	else
		return true
	end
end

function HideBlizzard:GetModuleEnabled(module)
	return self.db.profile.modules[module]
end

function HideBlizzard:SetModuleEnabled(module, value)
	local old = HideBlizzard.db.profile.modules[module]
	HideBlizzard.db.profile.modules[module] = value
	if old ~= value then
		if value then
			HideBlizzard:EnableModule(module)
		else
			HideBlizzard:DisableModule(module)
		end
	end
end