local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
local MODNAME = "Action Bar"
local ActionBar = HideBlizzard:NewModule(MODNAME)

local db
local defaults = {
	profile = {
		["*"] = false,
	},
}

local optGet, optSet
do
	local mod = ActionBar
	function optGet(info)
		local key = info[#info]
		return db[key]
	end

	function optSet(info, value)
		local key = info[#info]
		db[key] = value
		mod:UpdateView()
	end
end

local options
local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "ActionBar",
			arg = MODNAME,
			get = optGet,
			set = optSet,
			args = {
				enabled = {
					type = "toggle",
					name = "Enable ActionBar",
					descStyle = "inline",
					desc = "ActionBar module hides frames connected to the main action bar at the bottom of the screen",
					width = "full",
					get = function() return HideBlizzard:GetModuleEnabled(MODNAME) end,
					set = function(info, value) HideBlizzard:SetModuleEnabled(MODNAME, value) end,
				},
				gryphons = {
					type = "toggle",
					name = "Gryphons",
					desc = "Hides the art on the ends of the main action bar",
					width = "full",
					disabled = function() return not ActionBar:IsEnabled() or db.mainmenubar end,
				},
				mainmenubar = {
					type = "toggle",
					name = "Main Menu Bar",
					desc = "Hides the main action bar and the frames connected to it",
					width = "full",
					disabled = function() return not ActionBar:IsEnabled() end,
				},
				repbar = {
					type = "toggle",
					name = "Rep Bar",
					desc = "Hides the reputation bar",
					width = "full",
					disabled = function() return not ActionBar:IsEnabled() or db.mainmenubar end,
				},
				stancebar = {
					type = "toggle",
					name = "Stance Bar",
					desc = "Hides the stance/shapeshift bar",
					width = "full",
					disabled = function() return not ActionBar:IsEnabled() or db.mainmenubar or GetNumShapeshiftForms() == 0 end,
				},
				xpbar = {
					type = "toggle",
					name = "XP Bar",
					desc = "Hides the experience bar",
					width = "full",
					disabled = function() return not ActionBar:IsEnabled() or db.mainmenubar or UnitLevel("player") == MAX_PLAYER_LEVEL end,
				},
			},
		}
	end

	return options
end

function ActionBar:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(HideBlizzard:GetModuleEnabled(MODNAME))
	HideBlizzard:RegisterModuleOptions(MODNAME, optSettings, "ActionBar")
end

function ActionBar:OnEnable()
	self:UpdateView()
end

function ActionBar:OnDisable()
	self:UpdateView()
end

function ActionBar:UpdateView()
	db = self.db.profile

	if db.gryphons == true and ActionBar:IsEnabled() then
		MainMenuBarLeftEndCap:Hide()
		MainMenuBarRightEndCap:Hide()
	else
		MainMenuBarLeftEndCap:Show()
		MainMenuBarRightEndCap:Show()
	end

	if db.mainmenubar == true and ActionBar:IsEnabled() then
		MainMenuBar:UnregisterAllEvents()
		MainMenuBar:Hide()
		MainMenuBar.Show = function() end
	else
		MainMenuBar.Show = nil
		MainMenuBar:Show()
		MainMenuBar:RegisterEvent("PLAYER_ENTERING_WORLD")
		MainMenuBar:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
		MainMenuBar:RegisterEvent("ADDON_LOADED")
		MainMenuBar:RegisterEvent("BAG_UPDATE")
		MainMenuBar:RegisterEvent("UNIT_ENTERING_VEHICLE")
		MainMenuBar:RegisterEvent("UNIT_ENTERED_VEHICLE")
		MainMenuBar:RegisterEvent("UNIT_EXITING_VEHICLE")
		MainMenuBar:RegisterEvent("UNIT_EXITED_VEHICLE")
	end

	if db.repbar == true and ActionBar:IsEnabled() then
		for factionIndex = 1, GetNumFactions() do
			local isWatched = GetFactionInfo(factionIndex)
			if isWatched then
				ReputationWatchBar:Hide()
				ReputationWatchBar.Show = function() end
				ReputationWatchBar_Update()
			else
				ReputationWatchBar:Hide()
				ReputationWatchBar.Show = function() end
				ReputationWatchBar_Update()
			end
		end
	else
		for factionIndex = 1, GetNumFactions() do
			local isWatched = GetFactionInfo(factionIndex)
			if isWatched then
				ReputationWatchBar.Show = nil
				ReputationWatchBar:Show()
				ReputationWatchBar_Update()
			else
				ReputationWatchBar:Hide()
				ReputationWatchBar.Show = function() end
				ReputationWatchBar_Update()
			end
		end
	end

	if db.stancebar == true and ActionBar:IsEnabled() then
		if GetNumShapeshiftForms() ~= 0 then
			ShapeshiftBarFrame:Hide()
			ShapeshiftBarFrame.Show = function() end
		end
	else
		if GetNumShapeshiftForms() ~= 0 then
			ShapeshiftBarFrame.Show = nil
			ShapeshiftBarFrame:Show()
		end
	end

	if db.xpbar == true and ActionBar:IsEnabled() then
		MainMenuExpBar:Hide()
		MainMenuExpBar.Show = function() end
		MainMenuExpBar_Update()
	else
		MainMenuExpBar.Show = nil
		MainMenuExpBar:Show()
		MainMenuExpBar_Update()
	end
end