local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
local MODNAME = "Buttons"
local Buttons = HideBlizzard:NewModule(MODNAME)

local db
local defaults = {
	profile = {
		["*"] = false,
	},
}

local optGet, optSet
do
	local mod = Buttons
	function optGet(info)
		local key = info[#info]
		return db[key]
	end

	function optSet(info, value)
		local key = info[#info]
		db[key] = value
		mod:UpdateView()
	end
end

local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "Buttons",
			arg = MODNAME,
			get = optGet,
			set = optSet,
			args = {
				enabled = {
					type = "toggle",
					order = 1,
					name = "Enable Buttons",
					descStyle = "inline",
					desc = "Buttons module hides buttons connected to the chat and minimap frame",
					width = "full",
					get = function() return HideBlizzard:GetModuleEnabled(MODNAME) end,
					set = function(info, value) HideBlizzard:SetModuleEnabled(MODNAME, value) end,
				},
				chatbuttons = {
					type = "toggle",
					order = 2,
					name = "Chat Buttons",
					desc = "Hides the chat arrows on the chat frame",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				battleground = {
					type = "toggle",
					order = 3,
					name = "Battleground",
					desc = "Hides the battleground queue button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				calendar = {
					type = "toggle",
					order = 4,
					name = "Calendar",
					desc = "Hides the calendar button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				mail = {
					type = "toggle",
					order = 5,
					name = "Mail",
					desc = "Hides the mail button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				tracking = {
					type = "toggle",
					order = 6,
					name = "Tracking",
					desc = "Hides the tracking button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				voice = {
					type = "toggle",
					order = 7,
					name = "Voice",
					desc = "Hides the voice button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				world = {
					type = "toggle",
					order = 8,
					name = "World Map",
					desc = "Hides the world map button",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
				zoom = {
					type = "toggle",
					order = 9,
					name = "Zoom",
					desc = "Hides both the zoom buttons",
					width = "full",
					disabled = function() return not Buttons:IsEnabled() end,
				},
			},
		}
	end

	return options
end

function Buttons:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(HideBlizzard:GetModuleEnabled(MODNAME))
	HideBlizzard:RegisterModuleOptions(MODNAME, optSettings, "Buttons")
end

function Buttons:OnEnable()
	self:UpdateView()
end

function Buttons:OnDisable()
	self:UpdateView()
end

function Buttons:UpdateView()
	db = self.db.profile

	if db.chatbuttons == true and Buttons:IsEnabled() then
		ChatFrameMenuButton:Hide()
		for i = 1, NUM_CHAT_WINDOWS do
			local up = _G["ChatFrame"..i.."UpButton"]
			up:Hide()
			up.Show = function() end
			local down = _G["ChatFrame"..i.."DownButton"]
			down:Hide()
			down.Show = function() end
			local bottom = _G["ChatFrame"..i.."BottomButton"]
			bottom:Hide()
			bottom.Show = function() end
		end
	else
		ChatFrameMenuButton:Show()
		for i = 1, NUM_CHAT_WINDOWS do
			local up = _G["ChatFrame"..i.."UpButton"]
			up.Show = nil
			up:Show()
			local down = _G["ChatFrame"..i.."DownButton"]
			down.Show = nil
			down:Show()
			local bottom = _G["ChatFrame"..i.."BottomButton"]
			bottom.Show = nil
			bottom:Show()
		end
	end

	if db.battleground == true and Buttons:IsEnabled() then
		MiniMapBattlefieldFrame:UnregisterAllEvents()
		MiniMapBattlefieldFrame:Hide()
	else
		BattlefieldFrame_UpdateStatus()
	end

	if db.calendar == true and Buttons:IsEnabled() then
		GameTimeFrame:UnregisterAllEvents()
		GameTimeFrame:Hide()
	else
		GameTimeFrame:Show()
		GameTimeFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
		GameTimeFrame:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES")
		GameTimeFrame:RegisterEvent("CALENDAR_EVENT_ALARM")
	end

	if db.mail == true and Buttons:IsEnabled() then
		MiniMapMailFrame:UnregisterAllEvents()
		MiniMapMailFrame:Hide()
	else
		if HasNewMail() then
			MiniMapMailFrame:Show()
		else
			MiniMapMailFrame:Hide()
		end
	end

	if db.tracking == true and Buttons:IsEnabled() then
		MiniMapTracking:UnregisterAllEvents()
		MiniMapTrackingButton:Hide()
		MiniMapTrackingIcon:Hide()
		MiniMapTrackingBackground:Hide()
	else
		MiniMapTrackingButton:Show()
		MiniMapTrackingIcon:Show()
		MiniMapTrackingBackground:Show()
		MiniMapTrackingButton:RegisterEvent("MINIMAP_UPDATE_TRACKING")
	end

	if db.voice == true and Buttons:IsEnabled() then
		MiniMapVoiceChatFrame:UnregisterAllEvents()
		MiniMapVoiceChatFrame:Hide()
	else
		MiniMapVoiceChat_Update()
	end

	if db.world == true and Buttons:IsEnabled() then
		MiniMapWorldMapButton:Hide() 
	else
		MiniMapWorldMapButton:Show()
	end

	if db.zoom == true and Buttons:IsEnabled() then
		MinimapZoomIn:Hide()
		MinimapZoomOut:Hide()
	else
		MinimapZoomIn:Show()
		MinimapZoomOut:Show()
	end
end