local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
local MODNAME = "Player"
local Player = HideBlizzard:NewModule(MODNAME)

local db
local defaults = {
	profile = {
		["*"] = false,
	},
}

local optGet, optSet
do
	local mod = Player
	function optGet(info)
		local key = info[#info]
		return db[key]
	end

	function optSet(info, value)
		local key = info[#info]
		db[key] = value
		mod:UpdateView()
	end
end

local options
local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "Player",
			arg = MODNAME,
			get = optGet,
			set = optSet,
			args = {
				enabled = {
					type = "toggle",
					order = 1,
					name = "Enable Player",
					descStyle = "inline",
					desc = "Player module hides player related frames",
					width = "full",
					get = function() return HideBlizzard:GetModuleEnabled(MODNAME) end,
					set = function(info, value) HideBlizzard:SetModuleEnabled(MODNAME, value) end,
				},
				armoredman = {
					type = "toggle",
					order = 2,
					name = "Armored Man",
					desc = "Hides the armored man under the minimap",
					width = "full",
					disabled = function() return not Player:IsEnabled() end,
				},
				aura = {
					type = "toggle",
					order = 3,
					name = "Aura",
					desc = "Hides the buff and debuff frame",
					width = "full",
					disabled = function() return not Player:IsEnabled()  end,
				},
				druidmanabar = {
					type = "toggle",
					order = 4,
					name = "Druid Mana Bar",
					desc = "Hides the druid mana bar when you shapeshift",
					width = "full",
					disabled = function() return not Player:IsEnabled()  or HideBlizzard:CheckClass("DRUID") end,
				},
				playercastbar = {
					type = "toggle",
					order = 5,
					name = "Player Cast Bar",
					desc = "Hides the player cast bar",
					width = "full",
					disabled = function() return not Player:IsEnabled()  end,
				},
				playerunitframe = {
					type = "toggle",
					order = 6,
					name = "Player Unit Frame",
					desc = "Hides the player unit frame",
					width = "full",
					disabled = function() return not Player:IsEnabled()  end,
				},
				runeframe = {
					type = "toggle",
					order = 7,
					name = "Rune Frame",
					desc = "Hides the rune frame",
					width = "full",
					disabled = function() return not Player:IsEnabled()  or HideBlizzard:CheckClass("DEATHKNIGHT") end,
				},
				totemframe = {
					type = "toggle",
					order = 8,
					name = "Totem Frame",
					desc = "Hides the totem frame",
					width = "full",
					disabled = function() return not HideBlizzard.db.profile.modules.Player or HideBlizzard:CheckClass("SHAMAN") end,
				},
			},
		}
	end

	return options
end

function Player:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(HideBlizzard:GetModuleEnabled(MODNAME))
	HideBlizzard:RegisterModuleOptions(MODNAME, optSettings, "Player")
end

function Player:OnEnable()
	self:UpdateView()
end

function Player:OnDisable()
	self:UpdateView()
end

function Player:UpdateView()
	db = self.db.profile

	if db.armoredman == true and Player:IsEnabled() then
		DurabilityFrame:UnregisterAllEvents()
		DurabilityFrame:Hide() 
	else
		DurabilityFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")
		DurabilityFrame:RegisterEvent("UPDATE_INVENTORY_ALERTS")
	end

	if db.aura == true and Player:IsEnabled() then
		BuffFrame:UnregisterAllEvents()
		BuffFrame:Hide()
		TemporaryEnchantFrame:Hide()
	else
		TemporaryEnchantFrame:Show()
		BuffFrame:Show()
		BuffFrame:RegisterEvent("UNIT_AURA")
		BuffFrame:RegisterEvent("PLAYER_AURAS_CHANGED")
	end

	if db.druidmanabar == true and Player:IsEnabled() then
		for shapeshiftIndex = 1, GetNumShapeshiftForms() do
			local active = GetShapeshiftFormInfo(shapeshiftIndex)
			if active then
				PlayerFrameAlternateManaBar:UnregisterAllEvents()
				PlayerFrameAlternateManaBar:Hide()
				PlayerFrameAlternateManaBar.Show = function() end
			else
				PlayerFrameAlternateManaBar:UnregisterAllEvents()
				PlayerFrameAlternateManaBar:Hide()
				PlayerFrameAlternateManaBar.Show = function() end
			end
		end
	else
		for shapeshiftIndex = 1, GetNumShapeshiftForms() do
			local active = GetShapeshiftFormInfo(shapeshiftIndex)
			if active then
				PlayerFrameAlternateManaBar.Show = nil
				PlayerFrameAlternateManaBar:Show()
				PlayerFrameAlternateManaBar:RegisterEvent("PLAYER_ENTERING_WORLD")
				PlayerFrameAlternateManaBar:RegisterEvent("UNIT_DISPLAYPOWER")
				PlayerFrameAlternateManaBar:RegisterEvent("UNIT_MAXMANA")
				PlayerFrameAlternateManaBar:RegisterEvent("UNIT_MANA")
				AlternatePowerBar_UpdateValue(PlayerFrameAlternateManaBar)
				AlternatePowerBar_UpdateMaxValues(PlayerFrameAlternateManaBar)
				AlternatePowerBar_UpdatePowerType(PlayerFrameAlternateManaBar)
			else
				PlayerFrameAlternateManaBar:Hide()
				PlayerFrameAlternateManaBar.Show = function() end
			end
		end
	end

	if db.playercastbar == true and Player:IsEnabled() then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:Hide()
	else
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
		CastingBarFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	end

	if db.playerunitframe == true and Player:IsEnabled() then
		PlayerFrame:UnregisterAllEvents()
		PlayerFrameHealthBar:UnregisterAllEvents()
		PlayerFrameManaBar:UnregisterAllEvents()
		PlayerFrame:Hide()
	else
		PlayerFrame:RegisterEvent("UNIT_LEVEL")
		PlayerFrame:RegisterEvent("UNIT_COMBAT")
		PlayerFrame:RegisterEvent("UNIT_SPELLMISS")
		PlayerFrame:RegisterEvent("UNIT_PVP_UPDATE")
		PlayerFrame:RegisterEvent("UNIT_MAXMANA")
		PlayerFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
		PlayerFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
		PlayerFrame:RegisterEvent("PLAYER_UPDATE_RESTING")
		PlayerFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
		PlayerFrame:RegisterEvent("PARTY_LEADER_CHANGED")
		PlayerFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
		PlayerFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
		PlayerFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
		PlayerFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
		PlayerFrameHealthBar:RegisterEvent("UNIT_HEALTH")
		PlayerFrameHealthBar:RegisterEvent("UNIT_MAXHEALTH")
		PlayerFrameManaBar:RegisterEvent("UNIT_MANA")
		PlayerFrameManaBar:RegisterEvent("UNIT_RAGE")
		PlayerFrameManaBar:RegisterEvent("UNIT_FOCUS")
		PlayerFrameManaBar:RegisterEvent("UNIT_ENERGY")
		PlayerFrameManaBar:RegisterEvent("UNIT_RUNIC_POWER")
		PlayerFrameManaBar:RegisterEvent("UNIT_HAPPINESS")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXMANA")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXRAGE")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXFOCUS")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXENERGY")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXRUNIC_POWER")
		PlayerFrameManaBar:RegisterEvent("UNIT_MAXHAPPINESS")
		PlayerFrameManaBar:RegisterEvent("UNIT_RUNIC_POWER")
		PlayerFrameManaBar:RegisterEvent("UNIT_DISPLAYPOWER")
		PlayerFrame:RegisterEvent("UNIT_NAME_UPDATE")
		PlayerFrame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
		PlayerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
		PlayerFrame:Show()
	end
	if db.runeframe == true and Player:IsEnabled() then
		local _, class = UnitClass("player")
		if (class == "DEATHKNIGHT") then
			RuneFrame:UnregisterAllEvents()
			RuneFrame:Hide()
		end
	else
		local _, class = UnitClass("player")
		if (class == "DEATHKNIGHT") then
			RuneFrame:Show()
			RuneFrame:RegisterEvent("RUNE_POWER_UPDATE")
			RuneFrame:RegisterEvent("RUNE_TYPE_UPDATE")
			RuneFrame:RegisterEvent("UNIT_RUNIC_POWER")
			RuneFrame:RegisterEvent("UNIT_MAXRUNIC_POWER")
			RuneFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
		end
	end
	if db.totemframe == true and Player:IsEnabled() then
		local _, class = UnitClass("player")
		if (class == "SHAMAN") then
			for i = 1, MAX_TOTEMS do
				local totem = _G["TotemFrameTotem"..i]
				totem:UnregisterAllEvents()
				totem:Hide()
				totem.Show = function() end
			end
		end
	else
		local _, class = UnitClass("player")
		if (class == "SHAMAN") then
			for i = 1, MAX_TOTEMS do
				local totem = _G["TotemFrameTotem"..i]
				totem.Show = nil
				totem:RegisterEvent("PLAYER_TOTEM_UPDATE")
				totem:RegisterEvent("PLAYER_ENTERING_WORLD")
				totem:RegisterForClicks("RightButtonUp")
			end
		end
	end
end