local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
local MODNAME = "Target"
local Target = HideBlizzard:NewModule(MODNAME)

local db
local defaults = {
	profile = {
		["*"] = false,
	},
}

local optGet, optSet
do
	local mod = Target
	function optGet(info)
		local key = info[#info]
		return db[key]
	end

	function optSet(info, value)
		local key = info[#info]
		db[key] = value
		mod:UpdateView()
	end
end

local options
local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "Target",
			arg = MODNAME,
			get = optGet,
			set = optSet,
			args = {
				enabled = {
					type = "toggle",
					order = 1,
					name = "Enable Target",
					descStyle = "inline",
					desc = "Target module hides target related frames",
					width = "full",
					get = function() return HideBlizzard:GetModuleEnabled(MODNAME) end,
					set = function(info, value) HideBlizzard:SetModuleEnabled(MODNAME, value) end,
				},
				combopointframe = {
					type = "toggle",
					order = 2,
					name = "Combo Point Frame",
					desc = "Hides the combo point frame",
					width = "full",
					disabled = function() return not Target:IsEnabled() or db.targetunitframe end,
				},
				focusframe = {
					type = "toggle",
					order = 3,
					name = "Focus Frame",
					width = "full",
					desc = "Hides the focus frame",
					disabled = function() return not Target:IsEnabled() end,
				},
				targetunitframe = {
					type = "toggle",
					order = 4,
					name = "Target Unit Frame",
					desc = "Hides the target unit frame",
					width = "full",
					disabled = function() return not Target:IsEnabled() end,
				},
			},
		}
	end

	return options
end

function Target:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(HideBlizzard:GetModuleEnabled(MODNAME))
	HideBlizzard:RegisterModuleOptions(MODNAME, optSettings, "Target")
end

function Target:OnEnable()
	self:UpdateView()
end

function Target:OnDisable()
	self:UpdateView()
end

function Target:UpdateView()
	db = self.db.profile

	if db.combopointframe == true and Target:IsEnabled() then
		local _, class = UnitClass("player")
		if (class == "ROGUE") or (class =="DRUID") then
			for i = 1, MAX_COMBO_POINTS do
				local combopointframe = _G["ComboPoint"..i]
				combopointframe:Hide()
				combopointframe:UnregisterAllEvents()
				combopointframe.Show = function() end
			end
		end
	else
		local _, class = UnitClass("player")
		if (class == "ROGUE") or (class == "DRUID") then
			for i = 1, MAX_COMBO_POINTS do
				local combopointframe = _G["ComboPoint"..i]
				combopointframe.Show = nil
				combopointframe:Show()
				combopointframe:RegisterEvent("PLAYER_FOCUS_CHANGED")
				combopointframe:RegisterEvent("PLAYER_TARGET_CHANGED")
				combopointframe:RegisterEvent("PLAYER_COMBO_POINTS")
			end
		end
	end

	if db.focusframe == true and Target:IsEnabled() then
		FocusFrame:UnregisterAllEvents()
		FocusFrame:Hide()
	else
		if UnitExists("focus") then
			FocusFrame:Show()
		end
		FocusFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
		FocusFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
		FocusFrame:RegisterEvent("UNIT_HEALTH")
		FocusFrame:RegisterEvent("UNIT_LEVEL")
		FocusFrame:RegisterEvent("UNIT_FACTION")
		FocusFrame:RegisterEvent("UNIT_CLASSIFICATION_CHANGED")
		FocusFrame:RegisterEvent("UNIT_AURA")
		FocusFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
		FocusFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	end

	if db.targetunitframe == true and Target:IsEnabled() then
		TargetFrame:UnregisterAllEvents()
		TargetFrame:Hide()
	else
		if UnitExists("target") then
			TargetFrame:Show()
		end
		TargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
		TargetFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
		TargetFrame:RegisterEvent("UNIT_HEALTH")
		TargetFrame:RegisterEvent("UNIT_LEVEL")
		TargetFrame:RegisterEvent("UNIT_FACTION")
		TargetFrame:RegisterEvent("UNIT_CLASSIFICATION_CHANGED")
		TargetFrame:RegisterEvent("UNIT_AURA")
		TargetFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
		TargetFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	end
end