local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
local MODNAME = "Vehicle"
local Vehicle = HideBlizzard:NewModule(MODNAME)

local db
local defaults = {
	profile = {
		["*"] = false,
	},
}

local optGet, optSet
do
	local mod = ActionBar
	function optGet(info)
		local key = info[#info]
		return db[key]
	end

	function optSet(info, value)
		local key = info[#info]
		db[key] = value
		mod:UpdateView()
	end
end

local options
local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "Vehicle",
			arg = MODNAME,
			get = optGet,
			set = optSet,
			args = {
				enabled = {
					type = "toggle",
					name = "Enable Vehicle",
					descStyle = "inline",
					desc = "Vehicle module hides vehicle related frames",
					width = "full",
					get = function() return HideBlizzard:GetModuleEnabled(MODNAME) end,
					set = function(info, value) HideBlizzard:SetModuleEnabled(MODNAME, value) end,
				},
				vehiclemenubar = {
					type = "toggle",
					name = "Vehicle Menu Bar",
					desc = "Hides the vehicle menu bar",
					width = "full",
					disabled = function() return not Vehicle:IsEnabled() end,
				},
				vehicleseat = {
					type = "toggle",
					name = "Vehicle Seat",
					desc = "Hides the vehicle seat under the minimap",
					width = "full",
					disabled = function() return not Vehicle:IsEnabled() end,
				},
			},
		}
	end

	return options
end

function Vehicle:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(HideBlizzard:GetModuleEnabled(MODNAME))
	HideBlizzard:RegisterModuleOptions(MODNAME, optSettings, "Vehicle")
end

function Vehicle:OnEnable()
	self:UpdateView()
end

function Vehicle:OnDisable()
	self:UpdateView()
end

function Vehicle:UpdateView()
	db = self.db.profile

	if db.vehiclemenubar == true and Vehicle:IsEnabled() then
		VehicleMenuBar:UnregisterAllEvents()
		VehicleMenuBar:Hide()
		VehicleMenuBar.Show = function() end
	else
		if UnitInVehicle("player") then
			VehicleMenuBar.Show = nil
			VehicleMenuBar:Show()
		end
		VehicleMenuBar:RegisterEvent("UNIT_ENTERING_VEHICLE")
		VehicleMenuBar:RegisterEvent("UNIT_ENTERED_VEHICLE")
		VehicleMenuBar:RegisterEvent("UNIT_EXITED_VEHICLE")
		VehicleMenuBar:RegisterEvent("UNIT_DISPLAYPOWER")
		VehicleMenuBar:RegisterEvent("VEHICLE_ANGLE_UPDATE")
		VehicleMenuBar:RegisterEvent("VEHICLE_PASSENGERS_CHANGED")
		VehicleMenuBar:RegisterEvent("PLAYER_GAINS_VEHICLE_DATA")
		VehicleMenuBar:RegisterEvent("PLAYER_LOSES_VEHICLE_DATA")
		VehicleMenuBar:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
	if db.vehicleseat == true and Vehicle:IsEnabled() then
		VehicleSeatIndicator:UnregisterAllEvents()
		VehicleSeatIndicator:Hide()
		VehicleSeatIndicator.Show = function() end
	else
		if UnitInVehicle("player") then
			VehicleSeatIndicator.Show = nil
			VehicleSeatIndicator:Show()
		end
		VehicleSeatIndicator:RegisterEvent("UNIT_ENTERING_VEHICLE")
		VehicleSeatIndicator:RegisterEvent("UNIT_ENTERED_VEHICLE")
		VehicleSeatIndicator:RegisterEvent("UNIT_EXITED_VEHICLE")
		VehicleSeatIndicator:RegisterEvent("UNIT_DISPLAYPOWER")
		VehicleSeatIndicator:RegisterEvent("VEHICLE_ANGLE_UPDATE")
		VehicleSeatIndicator:RegisterEvent("VEHICLE_PASSENGERS_CHANGED")
		VehicleSeatIndicator:RegisterEvent("PLAYER_GAINS_VEHICLE_DATA")
		VehicleSeatIndicator:RegisterEvent("PLAYER_LOSES_VEHICLE_DATA")
		VehicleSeatIndicator:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
end