local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")

local optGet, optSet
do
	function optGet(info)
		local key = info[#info] 
		return HideBlizzard.db.profile[key]
	end
	
	function optSet(info, value)
		local key = info[#info]
		HideBlizzard.db.profile[key] = value
		HideBlizzard:UpdateView()
	end
end

local options, moduleOptions = nil, {}
local function optSettings()
	if not options then
		options = {
			type = "group",
			name = "HideBlizzard",
			args = {
				main = {
					type = "group",
					name = "HideBlizzard",
					get = optGet,
					set = optSet,
					args = {
						intro = {
							type = "description",
							order = 1,
							name = "HideBlizzard lets you hide blizzard frames located on the interface. You can toggle any of the available frames listed to hide them. The settings are saved per-character so you can have individual settings for each of them.\n\n|cffff0000HideBlizzard was intended to be used with addons that dont modify blizzard frames or without the use of any other addon(s) and cannot take responsibility with incompatibility issues that may surface from them.|r\n",
						},
						about = {
							type = "execute",
							order = 2,
							name = "About",
							desc = "View the about panel",
							func = function() InterfaceOptionsFrame_OpenToCategory(HideBlizzard.optionFrame.About) end,
						},
						changelog = {
							type = "execute",
							order = 3,
							name = "Changelog",
							desc = "View the changes in HideBlizzard",
							func = function() HideBlizzard:ShowChangelog() end,
						},
						help = {
							type = "execute",
							order = 4,
							name = "Help!",
							desc = "Open the help dialog",
							func = function() HideBlizzard:ShowHelp()
							end,
						},
						reset = {
							type = "execute",
							order = 5,
							name = "Reset",
							desc = "Reset settings to defaults",
							func = function() StaticPopup_Show("RESET_CONFIRM") end,
						},
					},
				},
			},
		}
		for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v
		end
	end

	return options
end

function HideBlizzard:RegisterOptions()
	self.optionFrame = {}

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HideBlizzard", optSettings)
	self.optionFrame.HideBlizzard = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideBlizzard", nil, nil, "main")
	self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles")
end

function HideBlizzard:RegisterModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl
	self.optionFrame[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideBlizzard", displayName, "HideBlizzard", name)
end